/*
    Fichier messages.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Bibliothque de fonctions utilisant des fichiers de donnes
    pour dialoguer avec l'utilisateur dans sa langue.
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "messages.h"
#include "szchemin.h"


// #define DEBUG


#define  szlig   120  // longueur maximale des lignes des fichiers de donnes



/* variables globales au source */

char *ncom;  // nom de (ou chemin d'accs ) la commande en cours d'excution
char repert_com [szchemin] = ""; // rpertoire o est implante cette commande



/* mmorise le nom ou le chemin d'accs de la commande en cours d'excution */

void memcom (char *arg0)
{
    ncom = arg0;
}



/* retourne le nom ou le chemin d'accs de la commande en cours d'excution */

char *nomcom ()
{
    return (ncom);
}



/* retourne le nom du rpertoire o est la commande en cours d'excution */

char *dircom ()
{
    int derslash;  // position du dernier slash rencontr dans ncom
    char *path;    // variable d'environnement PATH
    char chemcom [szchemin]; // chemin d'accs complet  la commande
    int  i, j;     // indices pour parcourir et recopier ncom et path


    // si le nom de ce rpertoire n'est pas encore mmoris
    if (! *repert_com)
    {
        // initialisation
        derslash = 0;
        i = 0;

        // recopier ncom dans repert_com et chercher la position du dernier /
        while (ncom [i])
        {
            if (ncom [i] == '/')
                derslash = i;

            repert_com [i++] = ncom [i];
        }

        // on ne conserve dans repert_com que le nom du rpertoire
        repert_com [derslash] = '\0';

        // si aucun nom de rpertoire n'a t trouv
        if (! *repert_com)
        {
            // on va tester les chemins de la variable d'environnement PATH
            path = getenv ("PATH");

            i = 0;

            do
            {
                j = 0;

                // recopier l'un de ces chemins
                while (path [i] != ':' && path [i])
                {
                    repert_com [j++] = path [i++];
                }

                // terminer la chaine de caractres
                repert_com [j] = '\0';

                // gnrer le chemin d'accs  la commande
                sprintf (chemcom, "%s/%s", repert_com, ncom);
            }
            // termin si ce chemin d'accs est le bon
            while (access (chemcom, 0) != 0 && path [i++]);
        }
    }

    // retourner le nom du rpertoire
    return (repert_com);
}



/* retourne le nom du fichier ou du rpertoire associ au mnmonique */

char *ficdir  (char *mnemonique)
{
    static int testacces = 0;    // mmorise si on a dj appel cette fonction
    char chem_ficdir [szchemin]; // chemin d'accs au fichier ficdir-libremail
    static FILE *desc_ficdir;    // descripteur du fichier ficdir-libremail
    char ligne [szlig];          // contenu d'une ligne du fichier
    static char nom_ficdir [szmax_ficdir]; // nom associ au mnmonique
    int  i, j;                   // indices pour traitements chaines caractres


#ifdef DEBUG
    printf ("Appel de ficdir (%s)\n", mnemonique);
    sleep (1);
#endif

    // si on appelle cette fonction pour la premire fois
    if (! testacces)
    {
        // gnrer le chemin d'accs au fichier ficdir-libremail
        sprintf (chem_ficdir, "%s/ficdir-libremail", dircom ());

        // tenter de l'ouvrir en lecture
        desc_ficdir = fopen (chem_ficdir, "r");

        // on ne repassera plus dans ce bloc d'instructions
        testacces = 1;
    }

    // si le fichier ficdir-libremail a pu tre ouvert
    if (desc_ficdir)
    {
        // se positionner au dbut de ce fichier
        rewind (desc_ficdir);

        // on va l'explorer pour trouver le mnmonique
        while (fgets (ligne, szlig, desc_ficdir))
        {
            // comparaison du mnmonique de la ligne
            // lue avec celui pass en paramtre
            i = 0;

            while (ligne [i] == mnemonique [i])
                i++;

            // si mnmonique trouv
            if (mnemonique [i] == '\0')
            {
                // se positionner sur le nom correspondant
                while (ligne [i] < 'a' && ligne [i])
                    i++;

                j = 0;

                // on va recopier ce nom
                while (ligne [i] > '#')
                {
                    nom_ficdir [j++] = ligne [i++];

                    // en vitant un dbordement de tableau
                    if (j >= szmax_ficdir)
                // "Lors de l'appel de ficdir (%s), le nom obtenu est trop long"
                        errfatale ("DEBORDE_FICDIR", mnemonique);
                }

                // terminer le nom
                nom_ficdir [j] = '\0';

                // retourner le nom trouv (s'il n'est pas vide !)
                if (*nom_ficdir)
                    return (nom_ficdir);
            }
        }
    }

    // mnmonique non trouv dans un fichier, on va gnrer le nom par dfaut

    // initialisation compteurs
    i = 0;
    j = 0;

    // sauter le prfixe DIR_ ou FIC_ du mnmonique
    while (mnemonique [i++] != '_' && mnemonique [i])
        ;   // passage au caractre suivant

    // recopier la fin du mnmonique
    do
    {
        // en vitant un dbordement de tableau
        if (j >= szmax_ficdir)
            // "Lors de l'appel de ficdir (%s), le nom obtenu est trop long"
            errfatale ("DEBORDE_FICDIR", mnemonique);

        // en passant les caractres en minuscules
        nom_ficdir [j++] = tolower (mnemonique [i]);
    }
    while (mnemonique [i++]);

    // retourner le nom que l'on vient de gnrer
    return (nom_ficdir);
}



/* sortie du programme sur impossibilit d'afficher un message */

void erreur_mess (int numerr, char *info)
{
    switch (numerr)
    {
        case 1: fprintf (stderr, "Fichier %s manquant ou protg en lecture",
                                                                        info);
                break;

        case 2: fputs ("Mnmonique MNEMO_ABSENT manquant dans le fichier des messages",
                                                                      stderr);
                break;
    }

    // passer  la ligne
    fputc ('\n', stderr);

    // sortir du programme
    exit (-1);
}



/* retourne le message associ au mnmonique pass en paramtre */

char *message (char *mnemonique)
{
    static int testacces = 0;    // mmorise si on a dj appel cette fonction
    int  retourdeb;              // mmorise si retour en dbut de fichier
    char ficmes [szchemin];      // chemin d'accs au fichier mess-libremail
    static FILE *desc_ficmes;    // descripteur du fichier mess-libremail
    char ligne [szlig];          // contenu d'une ligne du fichier
    static char mess_lu [szlig]; // message rcupr dans le fichier
    int  i, j;                   // indices pour traitements chaines caractres


#ifdef DEBUG
    printf ("Appel de message (%s)\n", mnemonique);
    sleep (1);
#endif

    // si on appelle cette fonction pour la premire fois
    if (! testacces)
    {
        // gnrer le chemin d'accs au fichier ficmes-libremail
        sprintf (ficmes, "%s/mess-libremail", dircom ());

        // tenter de l'ouvrir en lecture
        desc_ficmes = fopen (ficmes, "r");

        // si le fichier mess-libremail n'a pas pu tre ouvert
        if (!desc_ficmes)
            // message d'erreur fatale
            // "Fichier %s manquant ou protg en lecture"
            erreur_mess (1, ficmes);

        // on ne repassera plus dans ce bloc d'instructions
        testacces = 1;

        // on est au dbut du fichier
        retourdeb = 1;
    }
    // sinon
    else
        // on va commencer par tester la chaine qui suit la dernire trouve
        retourdeb = 0;

    // le fichier ficmes-libremail a pu tre ouvert
    // on va l'explorer pour trouver le mnmonique

    // premier essai  partir de la ligne qui suit celle du message prcdent
    fgets (ligne, szlig, desc_ficmes);

    do
    {
        // comparaison du mnmonique de la ligne
        // lue avec celui pass en paramtre
        i = 0;

        while (ligne [i] == mnemonique [i])
            i++;

        // si mnmonique trouv
        if (mnemonique [i] == '\0' && (ligne [i] == ' ' || ligne [i] == '\t'))
        {
            // se positionner sur le message correspondant
            while (ligne [i] != '"' && ligne [i])
                i++;

            j = 0;
            i++;

            // on va recopier le message
            while (ligne [i] != '"' && ligne [i])
            {
                // si \ dans le message, on convertit le caractre spcial
                if (ligne [i] == '\\')
                {
                    switch (ligne [++i])
                    {
                        case 'n' : mess_lu [j++] = '\n';
                                   break;

                        case 'r' : mess_lu [j++] = '\r';
                                   break;

                        case 't' : mess_lu [j++] = '\t';
                                   break;

                        case '\\': mess_lu [j++] = '\\';
                                   break;

                        default  : mess_lu [j++] = '\\';
                                   mess_lu [j++] = ligne [i];
                    }

                    // et on passe au caractre suivant
                    i++;
                }
                // sinon, simple copie du caractre
                else
                    mess_lu [j++] = ligne [i++];
            }

            // terminer le message
            mess_lu [j] = '\0';

            // retourner le message trouv
            return (mess_lu);
        }

        // si on vient de tester sans succs la chaine du fichier
        // qui suit celle du message demand juste auparavant
        if (! retourdeb)
        {
            // reprendre la recherche en dbut de fichier
            rewind (desc_ficmes);

            // on explorera le fichier jusqu' la fin si ncessaire
            retourdeb = 1;
        }
    }
    // continuer la recherche ligne par ligne jusqu'en fin de fichier
    while (fgets (ligne, szlig, desc_ficmes));

    // mnmonique non trouv dans le fichier mess-libremail
    if (strcmp ("MNEMO_ABSENT", mnemonique) != 0)
        // sortie sur message d'erreur
        errfatale ("MNEMO_ABSENT", mnemonique);
    else
        // "Mnmonique MNEMO_ABSENT manquant dans le fichier des messages"
        erreur_mess (2, NULL);
}



/* affichage d'une ligne de texte */

void affiche_msg (char *mnemonique)
{
    puts (message (mnemonique));
}



/* affichage d'un message d'erreur simple */

void affiche_err (char *mnemonique)
{
    fputs (message (mnemonique), stderr);
    fputc ('\n', stderr);
}



/* affichage d'un message d'erreur avec argument */

void aff_err_arg (char *mnemonique, char *argument)
{
    fprintf (stderr, message (mnemonique), argument);
    fputc ('\n', stderr);
}


/* sortie du programme sur erreur fatale */

void errfatale (char *mnemonique, char *argument)
{
    // crire le message d'erreur
    aff_err_arg (mnemonique, argument);

    // sortir du programme
    exit (-1);
}


/* sortie du programme en rappellant la syntaxe d'une commande */

void psyntaxe (char *mnemonique)
{
    errfatale (mnemonique, nomcom ());
}
