/*
    Fichier sjmails.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Ce programme affiche les caractristique de chaque mail
    en attente de tlchargement (ou de destruction).

    Il y a 3 formats d'affichage.

    Le format dtaill (option -d) donne plusieurs lignes d'information :
    - date et heure d'envoi
    - expditeur et destinataire(s) + ventuelle adresse de rponse
    - sujet du mail
    - format du message (texte, html ...)
    - ventuellement premires lignes du message

    Le format simplifi (option -s) affiche une ligne par message :
      nom expditeur     sujet     date et heure d'envoi

    La variante du format simplifi (option -e) affiche une ligne par message :
      adresse expditeur     sujet     date et heure d'envoi

    Un fichier de configuration est utilis pour se connecter  la
    boite aux lettres.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include "messages.h"
#include "buflect.h"
#include "ficonf.h"
#include "pop.h"
#include "trtentete.h"
#include "trtligne.h"


// largeur des champs affichs en mode simplifi (options -s et -e)
#define sz_nom         21
#define sz_adr         31
#define sz_avantdate   67

// options d'affichage
#define opt_s           0
#define opt_e           1
#define opt_d           2


/* prototypes */
void aff_detail (int numes);
void aff_resume (int numes, int lg_num);


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

int nblig_mes;    // nombre de lignes  afficher pour chaque message
int opt_aff;      // option affichage (simple = -s, dtaill = -d)


/* programme principal */

int main (int nbarg, char *varg[])
{
    FILE *fconf;         // descripteur du fichier de configuration
    int  numes, nbmes;   // numro du mail courant et nombre de mails
    int  lg_num;         // nombre caractres pour afficher tous numros mails
    int  sup_nbmes;      // variable pour calculer lg_num
    char format_numes [6]; // format affichage numros mails


    // rcupration du nom de l'excutable
    memcom (*varg);

    // se positionner sur le premier argument de sjmails
    varg ++;

    // initialisation options de fonctionnement
    nblig_mes = 0;

    // rcupration de l'option de fonctionnement ventuelle
    if (--nbarg > 1 && **varg == '-')
    {
        if (varg [0][1] == 's')
            opt_aff = opt_s;

        else if (varg [0][1] == 'e')
            opt_aff = opt_e;

        else if (varg [0][1] == 'd')
        {
            opt_aff = opt_d;

            // recupration du nombre de lignes s'il existe
            nblig_mes = atoi (varg [1]);

            // dans ce cas, dcompter l'argument trait
            if (nblig_mes || *varg [1] == '0')
            {
                varg ++;
                nbarg --;
            }
        }
        else
        {
            // "Option %s incorrecte"
            aff_err_arg ("ERR_OPTION", *varg);
            nbarg = 0;   // pour afficher la syntaxe de la commande
        }

        // dcompter l'argument trait
        varg ++;
        nbarg --;
    }
    else
        // option de fonctionnement par dfaut
        opt_aff = opt_s;

    // controle du nombre d'arguments
    if (nbarg == 1)
    {
        // ouvrir le fichier de configuration
        fconf = ouvre_ficonf (varg [0]);

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // rcupration du nombre de mails
                nbmes = nbmails ();

                // affichage dtaill ou rsum de l'entte des mails
                if (opt_aff == opt_d)
                {
                    for (numes = 1; numes <= nbmes; numes++)
                        aff_detail (numes);
                }
                else
                {
                    // auparavant calcul nbre min de car pour n messages
                    lg_num = 1;
                    sup_nbmes = 10;

                    while (nbmes >= sup_nbmes)
                    {
                        lg_num ++;
                        sup_nbmes *= 10;
                    }

                    // et fabrication de la chaine de format d'affichage
                    sprintf (format_numes, "%%%dd ", lg_num++);

                    // affichage rsum des enttes de mails
                    for (numes = 1; numes <= nbmes; numes++)
                    {
                        printf (format_numes, numes);
                        aff_resume (numes, lg_num);
                    }
                }

                // se dconnecter proprement du serveur pop
                deconnect_pop ();
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
    }
    else
        // "Syntaxe : %s [-(s|e|d [nb_lignes_messages])] fichier_configuration"
        psyntaxe ("SYNT_SJMAILS");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}



/* lit l'entte d'un message et affiche certaines lignes */

void aff_detail (int numes)
{
    char bufw [80]; // buffer d'envoi d'une requte
    // buffers pour affichage ordonn des caractristiques du message
    char bufFrom [80], bufTo [80], bufCc [80], bufReply [80],
         bufSubject [80], bufContent [80];
    int i, j;       // simples compteurs


    // initialisation
    bufFrom [0]    = '\0';
    bufTo [0]      = '\0';
    bufCc [0]      = '\0';
    bufReply [0]   = '\0';
    bufSubject [0] = '\0';
    bufContent [0] = '\0';

    // demande de lecture de l'entte du message
    if (nblig_mes)
        sprintf (bufw, "TOP %d %d", numes, nblig_mes);
    else
        sprintf (bufw, "TOP %d 1", numes);

    env_pop (bufw);

    // "Message %d\n"
    putchar ('\n');
    printf (message ("AFF_NUMAIL"), numes);

    // lecture et mmorisation des caractristiques du message
    do
    {
        // lire une ligne de l'entte du message
        lire_pop ();

        // si c'est le champ sujet
        if (start ("Subject"))
        {
            // mmoriser la ligne
            memconvbuf (bufSubject);

            // on va tester la ligne qui suit
            lire_pop ();
            majlignentete ();

            // si c'est encore une ligne du sujet
            if (*buf_lect == ' ' || *buf_lect == '\t')
            {
                // initialisation compteurs
                i = 1;
                j = strlen (bufSubject);

                // complter le sujet du mail
                while (buf_lect [i] && j < sizeof (bufSubject) - 1)
                    bufSubject [j++] = buf_lect [i++];

                // terminer la chaine de caractres
                bufSubject [j] = '\0';
            }
        }

        // mmorisation des autres caractristiques importantes
        if (start ("Date"))
            puts (buf_lect);   // la date est affiche directement
        else if (start ("From"))
            memconvbuf (bufFrom);
        else if (start ("To"))
            memconvbuf (bufTo);
        else if (start ("Cc"))
            memconvbuf (bufCc);
        else if (start ("Reply-To"))
            memconvbuf (bufReply);
        else if (start ("Content-Type"))
        {
            membuf (bufContent);

            if (recup_ctype () == TextPlain)
                mem_charset ();
        }
        else if (start ("Content-Transfer-Encoding"))
            mem_encodage ();
    }
    while (buf_lect [0] != '\0');  // lecture entte termine si ligne vide

    // affichage des caractristiques mmorises
    if (bufFrom [0])
        puts (bufFrom);
    if (bufTo [0])
        puts (bufTo);
    if (bufCc [0])
        puts (bufCc);
    if (bufReply [0])
        puts (bufReply);
    if (bufSubject [0])
        puts (bufSubject);
    if (bufContent [0])
        puts (bufContent);

    // sauter la ligne blanche qui prcde le message
    lire_pop ();

    // lecture de la fin du message jusqu'a une ligne limite a un .
    while (buf_lect [0] != '.' || buf_lect [1] != '\0')
    {
        // affichage ventuel des dernires lignes lues
        if (nblig_mes)
        {
            // on prend en compte les nouveaux champs Content ventuels
            if (start ("Content-Type") && recup_ctype () == TextPlain)
                mem_charset ();
            else if (start ("Content-Transfer-Encoding"))
                mem_encodage ();

            // convertir la ligne lue et l'afficher
            majligne ();
            printf ("%s", buf_lect);
        }

        // et lecture de la suivante
        lire_pop ();
    }
}



/* lit l'entte d'un message et affiche un rsum sur une ligne */

void aff_resume (int numes, int lg_num)
{
    char bufw [80]; // buffer d'envoi d'une requte
    // buffers pour affichage ordonn des caractristiques du message
    char bufFrom [80], bufSubject [80], bufDate [80];
    // 1re et ventuellement 3me lettre des noms de mois
    static char *mois [] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                             "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
    int sz_champ1;  // nombre max de caractres du champ expditeur
    int i, j;       // simples compteurs


    // initialisation
    bufFrom [0]    = '\0';
    bufSubject [0] = '\0';
    bufDate [0]    = '\0';

    // demande de lecture de l'entte du message
    if (nblig_mes)
        sprintf (bufw, "TOP %d %d", numes, nblig_mes);
    else
        sprintf (bufw, "TOP %d 1", numes);

    env_pop (bufw);

    // lecture et mmorisation des caractristiques du message
    do
    {
        // lire une ligne de l'entte du message
        lire_pop ();

        // si c'est le champ sujet
        if (start ("Subject"))
        {
            // mmoriser la ligne
            memconvbuf (bufSubject);

            // on va tester la ligne qui suit
            lire_pop ();
            majlignentete ();

            // si c'est encore une ligne du sujet
            if (*buf_lect == ' ' || *buf_lect == '\t')
            {
                // initialisation compteurs
                i = 1;
                j = strlen (bufSubject);

                // complter le sujet du mail
                while (buf_lect [i] && j < sizeof (bufSubject) - 1)
                    bufSubject [j++] = buf_lect [i++];

                // terminer la chaine de caractres
                bufSubject [j] = '\0';
            }
        }

        // mmorisation des autres caractristiques importantes
        if (start ("Date"))
            membuf (bufDate);
        else if (start ("From"))
            memconvbuf (bufFrom);
    }
    while (buf_lect [0] != '.' || buf_lect [1] != '\0');

    // initialisation : on est en dbut de ligne
    i = lg_num;

    // affichage du nom ou de l'adresse de l'expditeur du message
    if (*bufFrom)
    {
        j = 6;

        // option -s : affichage du nom de l'expditeur
        if (opt_aff == opt_s)
        {
            while (bufFrom [j] == '"' || bufFrom [j] == ' ')
                j++;

            putchar (bufFrom [j++]);

            while (++i < sz_nom && bufFrom [j] && bufFrom [j] != '<'
                         && bufFrom [j] != '"' && bufFrom [j] != '=')
                putchar (bufFrom [j++]);
        }
        // option -e : affichage de l'adresse de l'expditeur
        else
        {
            // recherche de la partie adresse de l'expditeur
            while (bufFrom [j] != '@' && bufFrom [j])
                j++;

            // positionnement au dbut de cette adresse
            while (bufFrom [j] != '<' && bufFrom [j] != ' ')
                j--;

            j++;

            // copie de l'adresse
            putchar (tolower (bufFrom [j++]));

            // pour ne pas tronquer l'adresse, on
            // accepte de dborder sur le champ sujet
            while (++i < sz_avantdate && bufFrom [j]
                                      && bufFrom [j] != '>')
                putchar (tolower (bufFrom [j++]));
        }
    }

    // dtermination nombre de caractres rserv au champ expditeur
    if (opt_aff == opt_s)
        sz_champ1 = sz_nom;
    else
        sz_champ1 = sz_adr;

    // complter ce champ par des blancs
    while (i++ <= sz_champ1)
        putchar (' ');

    // affichage du sujet du message
    if (*bufSubject)
    {
        j = 9;

        // on saute les blancs inutiles
        while (bufSubject [j] == ' ')
            j++;

        while (++i < sz_avantdate && bufSubject [j])
        {
            // si prsence d'une tabulation
            if (bufSubject [j] == '\t')
            {
                // afficher un blanc pour une meilleure mise en page
                putchar (' ');
                j++;
            }
            else
                // sinon affichage du caractre
                putchar (bufSubject [j++]);
        }
    }
    else
        i++;    // pour corriger l'alignement

    // complt par des blancs
    while (i++ <= sz_avantdate)
        putchar (' ');

    // affichage de la date et de l'heure du message
    if (*bufDate)
    {
        j = 6;

        // jour du mois
        while (bufDate [j] < '0' || bufDate [j] > '9')
            j++;

        printf ("%02d", atoi (bufDate + j));

        while (bufDate [j] > ' ')
            j++;

        j++;

        // mois
        i = 0;

        while (i < 12 && (toupper (bufDate [j]) != mois [i][0]
                     || tolower (bufDate [j+1]) != mois [i][1]
                     || tolower (bufDate [j+2]) != mois [i][2]))
            i++;

        printf ("/%02d/", i + 1);

        // anne sur 2 chiffres
        while (bufDate [j] < '0' || bufDate [j] > '9')
            j++;

        printf ("%02d", atoi (bufDate + j) % 100);

        while (bufDate [j] > ' ')
            j++;

        // heure, sans les secondes ni le fuseau horaire
        for (i = 0; i < 6; i++)
            putchar (bufDate [j++]);
    }

    // passage  la ligne
    putchar ('\n');
}
