/*
    Fichier vmailsj.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Ce programme affiche les caractristique de chaque mail
    mmoris dans un fichier.

    Pour chaque fichier mail, il y a une ligne d'affichage de la forme :
     expditeur      sujet     date et heure d'envoi
     destinataire    sujet     date et heure d'envoi
    selon qu'il s'agisse d'un mail reu ou envoy (ou  envoyer)

    On peut naviguer dans la liste avec les flches pour slectionner
    le mail  visualiser.
    Une fois slectionn, on pourra rpondre  ce mail ou le transfrer.

    Par dfaut, on analyse les mails du rpertoire courant.
    On peut aussi indiquer un autre rpertoire de recherche.

    Cet outil peut tre lanc automatiquement par la commande
    vmaildir lorsqu'on slectionne un rpertoire de l'arborescence
    des mails.
    Il lance  son tour vmailfic pour la visalisation des mails.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/stat.h>
#include "messages.h"
#include "buflect.h"
#include "fmail.h"
#include "trtentete.h"
#include "modepage.h"
#include "carspe.h"
#include "szchemin.h"


#define nbmaxfic     3000  // nombre maximum de fichiers mails traits

#define szmax_nom      25  // longueur max affichage du nom d'expditeur
#define szmax_sujet    65  // longueur max affichage sujet du mail
#define sz_date        15  // longueur affichage date et heure d'envoi
#define an_pivot       90  // l'anne des mails les plus vieux sur 2 chiffres


/* type de donnes */
typedef struct
{
    char nomfic [12];
    long ordre;
    char nom [szmax_nom];
    char sujet [szmax_sujet];
    char date [sz_date];
} descmail;


/* prototypes */
void lister_mails ();
int  memfic (char *nomfic);
void tridate ();
void navigation ();
void affligne (int numlig);
void suprefmail ();
void majliste_mails ();
void recherche (int sens);
int  dansligne (char *chaine, int numlig);


/* variable globale au source
   (pour viter des tonnes de passages de paramtres) */

descmail *fichmail [nbmaxfic];   // noms des fichiers mail


/* programme principal */

int main (int nbarg, char *varg[])
{
    // rcupration du nom de l'excutable
    memcom (*varg);

    // rcupration ventuelle du nom du rpertoire des mails
    if (--nbarg == 1)
    {
        // positionnement dans ce rpertoire
        if (chdir (varg [1]) < 0)
        {
            // "Rpertoire %s inexistant"
            aff_err_arg ("REPERT_INEXISTANT", varg [1]);
            return (0);
        }
    }

    // si aucune erreur dans la liste des paramtres
    if (nbarg < 2)
    {
        // rcupration de la liste des fichiers mail
        lister_mails ();

        // si des fichiers mails ont t trouvs
        if (nb_lignes > 0)
        {
            // les ordonner par date
            tridate ();

            // afficher leur caractristiques et naviguer dans la liste
            navigation ();
        }
        else
            // sinon, pas ncessaire de rester dans l'outil
            // "pas de mails dans ce rpertoire"
            affiche_msg ("REPERT_SANS_MAIL");
    }
    else
        // "Syntaxe : %s [rpertoire]"
        psyntaxe ("SYNT_GENE_REPERT");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}


/* rcupre les caractristiques des mails */

void lister_mails ()
{
    DIR    *repert;
    struct dirent *entree;
    struct stat   descfic;
    int    errmem;


    // initialisation
    nb_lignes = 0;
    errmem = 0;

    // accder au rpertoire
    repert = opendir (".");

    // lire un nom de fichier
    entree = readdir (repert);

    // tant qu'il y a des noms de fichier  lire et qu'on peut les traiter
    while (entree && ! errmem)
    {
        // mmoriser le fichier si son nom convient
        if (strlen (entree->d_name) == 8 ||
           (strlen (entree->d_name) == 10 && entree->d_name [8] == '.'))
        {
            // et si ce n'est pas un rpertoire
            stat (entree->d_name, &descfic);

            if ((descfic.st_mode & S_IFMT) == S_IFREG)
                errmem = memfic (entree->d_name);
        }

        // et lire le nom du fichier suivant
        entree = readdir (repert);
    }

    // avertissement si on a atteint les limites de stockage ou si problme
    if (errmem)
    {
        effpage ();

        if (errmem == 1)
        {
            // "Trop de fichier mail, liste tronque"
            // "Recompiler avec une valeur de nbmaxfic plus grande"
            affiche_msg ("DEBORD_NBMAXFIC-1");
            affiche_msg ("DEBORD_NBMAXFIC-2");
        }
        else
            // "Manque de place mmoire, liste des mails tronque"
            affiche_msg ("DEBORD_MEM_TRONCAT");

        sleep (3);
    }
}


/* mmorisation des caractristiques d'un mail */

int memfic (char *nomfic)
{
    // buffers pour mmoriser les caractristiques principales du message
    char bufFromTo [80], bufSubject [80], bufDate [80];

    // les noms de mois dans les champs Date: des mails
    static char *mois [] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                             "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

    descmail *mailcourant;  // la description en cours de gnration
    int jour, numois, an, heure, min;  // date et heure du mail
    int affichedest; // indique si on doit afficher le champ destinataire
    int i, j;        // simples compteurs


    // test du nombre de mails dj traits
    if (nb_lignes >= nbmaxfic)
        return (1);

    // ouverture du fichier mail
    fmail = fopen (nomfic, "r");

    // sortie si erreur imprvue
    if (fmail == 0)
    {
        affpage ();
        // "Fichier %s protg en lecture"
        aff_err_arg ("ACCES_FICH_LECT", nomfic);
        sleep (2);
        return (0);  // erreur non remonte
    }

    // dterminer le champ  afficher entre expditeur et destinataire
    affichedest = (tolower (*nomfic) == 'e') || (tolower (*nomfic) == 's');

    // initialisation
    bufFromTo [0]  = '\0';
    bufSubject [0] = '\0';
    bufDate [0]    = '\0';

    // lecture et mmorisation des caractristiques du message
    do
    {
        // lire une ligne de l'entte du message
        lire_fmail ();

        // si c'est le champ sujet
        if (start ("Subject"))
        {
            // mmoriser la ligne
            memconvbuf (bufSubject);

            // on va tester la ligne qui suit
            lire_fmail ();
            majlignentete ();

            // si c'est encore une ligne du sujet
            if (*buf_lect == ' ' || *buf_lect == '\t')
            {
                // initialisation compteurs
                i = 1;
                j = strlen (bufSubject);

                // complter le sujet du mail
                while (buf_lect [i] && j < sizeof (bufSubject) - 1)
                    bufSubject [j++] = buf_lect [i++];

                // terminer la chaine de caractres
                bufSubject [j] = '\0';
            }
        }

        // mmorisation des autres caractristiques importantes
        if (start ("Date"))
            membuf (bufDate);
        else if (start ("From") && ! affichedest)
            memconvbuf (bufFromTo);
        else if (start ("To") && affichedest)
            memconvbuf (bufFromTo);
    }
    while (buf_lect [0] != '\0');

    // librer le fichier mail
    fclose (fmail);

    // si l'on n'a pas trouv de ligne Date: From: ou Subject:
    if (! *bufDate && ! *bufFromTo && ! *bufSubject)
        return (0);   // ce n'est pas un fichier mail

    // allouer de l'espace mmoire pour stocker les caractristiques du mail
    mailcourant = (void *) malloc (sizeof (descmail));

    // sortir si manque de place mmoire
    if (! mailcourant)
        return (2);

    // mmoriser le nom du fichier
    strcpy (mailcourant->nomfic, nomfic);


    // mmorisation du nom de l'expditeur ou du destinataire du message
    j = 0;

    if (*bufFromTo)
    {
        // sauter le mot cl  From:  ou  To:
        if (affichedest)
            i = 4;
        else
            i = 6;

        // se positionner sur le 1er caractre significatif du nom
        while (bufFromTo [i] == '"' || bufFromTo [i] == ' ')
            i++;

        // mmorisation du nom
        mailcourant->nom [j++] = bufFromTo [i++];

        while (j < szmax_nom && bufFromTo [i] && bufFromTo [i] != '<'
                     && bufFromTo [i] != '"' && bufFromTo [i] != '=')
            mailcourant->nom [j++] = bufFromTo [i++];
    }

    // complter le nom par des blancs
    while (j < szmax_nom)
        mailcourant->nom [j++] = ' ';


    // mmorisation du sujet du message
    j = 0;

    if (*bufSubject)
    {
        // sauter le mot cl  Subject:
        i = 8;

        // se positionner sur le 1er caractre significatif du sujet
        while (bufSubject [i] == ' ')
            i++;

        // mmoriser le sujet
        while (j < szmax_sujet && bufSubject [i])
            mailcourant->sujet [j++] = bufSubject [i++];
    }

    // complter le sujet par des blancs
    while (j < szmax_sujet)
        mailcourant->sujet [j++] = ' ';


    // mmorisation de la date et de l'heure du message
    if (*bufDate)
    {
        // sauter le mot cl  Date:
        i = 6;

        // rcuprer le jour
        while (bufDate [i] < '0' || bufDate [i] > '9')
            i++;

        jour =  atoi (bufDate + i);

        // se positionner sur le mois
        while (bufDate [i] > ' ')
            i++;

        i++;

        // rcuprer le numro du mois
        numois = 11;

        while (numois >= 0 && (toupper (bufDate [i])  != mois [numois][0]
                           || tolower (bufDate [i+1]) != mois [numois][1]
                           || tolower (bufDate [i+2]) != mois [numois][2]))
            numois--;

        // rcuprer l'anne sur 2 chiffres
        while (bufDate [i] < '0' || bufDate [i] > '9')
            i++;

        an = atoi (bufDate + i) % 100;

        // rcuprer l'heure et les minutes
        while (bufDate [i] > ' ')
            i++;

        heure = atoi (bufDate + i);
        min = atoi (bufDate + i + 4);

        // mmoriser la date sous forme de chaine de caractres
        sprintf (mailcourant->date, "%02d/%02d/%02d %02d:%02d",
                                  jour, ++numois, an, heure, min);

        // les annes 90  99 sont considres du sicle pass
        if (an >= an_pivot)
            an = an - 2000;

        // mmoriser aussi la date sous forme numrique pour tri ultrieur
        /* on tient compte du rglage le plus frquent sur les ordinateurs
           europens : l'ordinateur est rgl sur l'heure locale et la
           valeur du fuseau horaire n'obit  aucune rgle stricte.
           Pour une utilisation dans des pays utilisant plusieurs fuseaux
           horaires, le rajout de celui ci pourra tre souhaitable. */

        mailcourant->ordre = ((((((((long) an * 12) + numois) * 31)
                                + jour) * 24) + heure) * 60) + min;
    }
    else
    {
        mailcourant->date [0] = '\0';
        mailcourant->ordre = 0;
    }


    // mmoriser la description dans la liste
    fichmail [nb_lignes++] = mailcourant;

    // tout s'est bien droul
    return (0);
}


/* tri des mails par date et heure */

void tridate ()
{
    descmail *tampon; // adresse d'une description de mail pour permutation
    int  i;           // indice dans la liste


    // initialisation
    i = 0;

    while (i < nb_lignes - 1)
    {
        // on compare 2 lments conscutifs
        if (fichmail [i]->ordre > fichmail [i+1]-> ordre)
        {
            // interversion de 2 enregistrements
            tampon = fichmail [i];
            fichmail [i] = fichmail [i+1];
            fichmail [i+1] = tampon;

            // ce qui peut supposer de comparer un nom dplac avec le prcdent
            if (i > 0)
                i--;
            else
                i++;
        }
        else
            i++;    // on progresse dans la liste
    }
}


/* affiche la liste des mails du rpertoire et permet de
   la parcourir mme si elle tient sur plusieurs pages */

void navigation ()
{
    int  car;          // caractre tap au clavier
    int  erreurs;      // nombre frappes caractres inconnus comme commandes
    int  choix;        // choix de ce qu'on veut imprimer
    char curdir [szchemin]; // rpertoire courant
    int  dans_mailenv; // indicateur : on est dans le rpertoire d'envoi
    char *editeur;     // diteur utilis pour modifier les mails
    // chaine de caractres utilise pour rebaptiser,
    // mettre  la poubelle ou imprimer un fichier mail
    char tampon [szchemin + 16];


    // configurer la liaison clavier pour lecture directe avec timeout
    mode_raw ();

    // initialisation
    lignecran = 1;
    lignecour = 0;
    erreurs = 0;
    affpage ();

    // rcuprer le nom du rpertoire courant
    getcwd (curdir, szchemin);

    // tester si on est dans le rpertoire d'envoi des mails
    if (getenv ("mailenv"))
        dans_mailenv = (strcmp (curdir, getenv ("mailenv")) == 0);
    else
        dans_mailenv = 0;

    do
    {
        // lire et traiter une touche du clavier
        car = leccar ();

        switch (car)
        {                  // dplacement dans la liste des mails
            case MONTE   : monte (1);
                           erreurs = 0;
                           break;

            case DESCEND : descend (1);
                           erreurs = 0;
                           break;

            case PAGEUP  : monte (lignepage + lignecran - 2);
                           erreurs = 0;
                           break;

            case PAGEDOWN: descend (2 * lignepage - lignecran - 2);
                           erreurs = 0;
                           break;

            case HOME    : monte (lignecour);
                           erreurs = 0;
                           break;

            case FIN     : descend (nb_lignes - lignecour - 1);
                           erreurs = 0;
                           break;

            case 0       : // aide si trop d'erreurs ou  la demande
            case F1      : effpage ();
                           // "Touches utilisables :\n"
                           // "flches Pageup, Pagedown, Home et Fin"
                           // "pour se dplacer d'une ou plusieurs lignes"
                           // "v ou l ou entre pour Voir (Lire) un mail"
                           // "r pour y Rpondre, t pour le Transfrer"
                           // "s pour le Supprimer, m pour Modifier avant envoi"
                           // "u pour restaurer un mail dans la poubelle"
                           // "j pour joindre des fichiers  un mail  envoyer"
                           // "n ou Inser pour en crer un Nouveau"
                           // "o pour voir le fichier Original (non converti)"
                           // "i pour Identifier le fichier mail"
                           // "p pour imPrimer un mail (ou la liste complte)"
                           // "/ ou ? pour rechercher une chaine dans la liste"
                           // "Control L pour rafficher la page"
                           // "Esc pour sortir de ce programme\n"
                           // "Appuyer sur une touche pour continuer"
                           affiche_msg ("AIDE_CHOIX_ADR-1");
                           affiche_msg ("AIDE_CHOIX_ADR-2");
                           affiche_msg ("AIDE_CHOIX_ADR-3");
                           affiche_msg ("AIDE_VMAILSJ-1");
                           affiche_msg ("AIDE_VMAILSJ-2");
                           affiche_msg ("AIDE_VMAILFIC-2");
                           affiche_msg ("AIDE_VMAILFIC-3");
                           affiche_msg ("AIDE_VMAILSJ-3");
                           affiche_msg ("AIDE_VMAILDIR_SJ");
                           affiche_msg ("AIDE_VMAILSJ-4");
                           affiche_msg ("AIDE_VMAILFIC_SJ");
                           affiche_msg ("AIDE_VMAILSJ-5");
                           affiche_msg ("AIDE_VMAILSJ-6");
                           affiche_msg ("AIDE_CHOIX_ADR-7");
                           affiche_msg ("AIDE_VMAIL");
                           affiche_msg ("ATTENTE_CLAVIER");
                           leccar ();
                           affpage ();
                           erreurs = 0;
                           break;

            case 'l'     :
            case 'v'     : // on va visualiser un mail
            case '\n'    : execom ("vmailfic", fichmail [lignecour]->nomfic);

                           // pas de surbrillance pour les mails dj visualiss
                           // (le test de la lettre  n  comme 10me caractre
                           // n'est utile que pour assurer la compatibilit en
                           // lecture avec Micro$oft Internet mail)
                           if (fichmail [lignecour]->nomfic [8] == '.'
                                && tolower(fichmail[lignecour]->nomfic[9])== 'n'
                                && access (fichmail [lignecour]->nomfic, 0) < 0)
                               fichmail [lignecour]->nomfic [8] = '\0';

                           // si le mail a t supprim
                           if (access (fichmail [lignecour]->nomfic, 0) < 0)
                               // on l'enlve de la liste  afficher
                               suprefmail ();

                           // on peut rafficher la page
            case CTRL    : affpage ();
                           erreurs = 0;
                           break;

                           // identifie le fichier mail (donne son nom)
            case 'i'     : effpage ();
                           // "\nFichier : %s/%s\n\n"
                           printf (message ("AFF_CHEMFICH"), curdir,
                                              fichmail [lignecour]->nomfic);

                           // "Appuyer sur une touche pour continuer"
                           printf (message ("ATTENTE_CLAVIER"));

                           leccar ();
                           affpage ();
                           erreurs = 0;
                           break;

                           // voir le fichier mail original sans conversion
            case 'o'     : execom ("less -r", fichmail [lignecour]->nomfic);
                           affpage ();
                           erreurs = 0;
                           break;

            case INSERT  : // cration d'un nouveau mail
            case 'n'     : execom ("cremail", "");

                           // si on est dans le rpertoire des mails  envoyer
                           if (dans_mailenv)
                               // ractualiser la liste des mails affichs
                               majliste_mails ();

                           affpage ();
                           erreurs = 0;
                           break;

                           // rponse  un mail
            case 'r'     : execom ("repmail", fichmail [lignecour]->nomfic);

                           // si on est dans le rpertoire des mails  envoyer
                           if (dans_mailenv)
                               // ractualiser la liste des mails affichs
                               majliste_mails ();

                           affpage ();
                           erreurs = 0;
                           break;

                           // transfert d'un mail
            case 't'     : execom ("trsfmail", fichmail [lignecour]->nomfic);

                           // si on est dans le rpertoire des mails  envoyer
                           if (dans_mailenv)
                               // ractualiser la liste des mails affichs
                               majliste_mails ();

                           affpage ();
                           erreurs = 0;
                           break;

      //    case SUPR    : // suppression d'un mail
            case 's'     : // rcuprer le rpertoire des mails supprims
                           if (getenv ("mailpoub"))
                               strcpy (tampon, getenv ("mailpoub"));
                           else
                               *tampon = '\0';

                           // si c'est le rpertoire courant
                           if (strcmp (curdir, tampon) == 0)
                           {
                               // supprimer le fichier mail
                               unlink (fichmail [lignecour]->nomfic);

                               // supprimer le mail de la liste  afficher
                               suprefmail ();
                           }
                           // sinon (cas gnral)
                           else if (*tampon)
                           {
                               // crer le rpertoire poubelle si ncessaire
                               mkdir (tampon, 0755);

                               // et y mettre le fichier mail
                               strcat (tampon, "/");
                               strcat (tampon, fichmail [lignecour]->nomfic);
                               rename (fichmail [lignecour]->nomfic, tampon);

                               // supprimer le mail de la liste  afficher
                               suprefmail ();
                           }
                           // sinon (racine des mails inconnue)
                           else
                           {
                               effpage ();
                            // "Racine des mails inconnue, pas de suppression"
                               affiche_msg ("REP_RACINE_INCONNU");
                               sleep (2);
                           }

                           affpage ();
                           erreurs = 0;
                           break;

                           // restauration d'un mail
            case 'u'     : // si on est dans le rpertoire poubelle
                           if (strcmp (curdir, getenv ("mailpoub")) == 0)
                           {
                               // dplacer le fichier mail dans entree
                               sprintf (tampon, "../%s/%s",
                          ficdir ("DIR_ENTREE"), fichmail [lignecour]->nomfic);
                               rename (fichmail [lignecour]->nomfic, tampon);

                               // supprimer le mail de la liste  afficher
                               suprefmail ();
                           }
                           // sinon, message d'erreur
                           else
                           {
                               effpage ();
                    // "Seuls les mails dans la poubelle peuvent tre restaurs"
                               affiche_msg ("MAIL_NON_RESTAUR");
                               sleep (2);
                           }

                           affpage ();
                           erreurs = 0;
                           break;

                           // modification d'un mail  envoyer
            case 'm'     : // si on est dans le rpertoire des mails  envoyer
                           if (dans_mailenv)
                           {
                               // modifier le mail
                               editeur = getenv ("EDITOR");

                               if (editeur)
                                   execom (editeur,fichmail[lignecour]->nomfic);
                               else
                                   execom ("vi", fichmail [lignecour]->nomfic);
                           }
                           else
                           {
                               // sinon, message d'erreur
                               effpage ();
                   // "Seuls les mails en attente d'envoi peuvent tre modifis"
                               affiche_msg ("MAIL_NON_MODIF");
                               sleep (2);
                           }

                           affpage ();
                           erreurs = 0;
                           break;

                           // impression de la liste ou du mail courant
            case 'p'     : // choix de ce qu'on veut imprimer
                           putchar ('\n');
                           effpage ();
                           // "1) impression de la liste des mails"
                           // "2) impression du mail de la ligne courante"
                           // "Votre choix ? "
                           affiche_msg ("CHOIX_IMPR-1");
                           affiche_msg ("CHOIX_IMPR-2");
                           printf (message ("VOTRE_CHOIX"));

                           choix = leccar () - '0';

                           // lancer la commande d'impression correspondante
                           if (choix == 1)
                               system ("sjfmails . | lpr");
                           else if (choix == 2)
                           {
                               sprintf (tampon, "voirfmail %s | lpr",
                                        fichmail [lignecour]->nomfic);
                               system (tampon);
                           }
                           else
                           {
                               // "Choix erronn"
                               affiche_msg ("CHOIX_IMPR_ERR");
                               sleep (2);
                           }

                           affpage ();
                           erreurs = 0;
                           break;

                           // rajout de fichiers joints  un mail  envoyer
            case 'j'     : // si on est dans le rpertoire des mails  envoyer
                           if (dans_mailenv)
                               // rajouter des fichiers joints
                               execom ("joindre", fichmail [lignecour]->nomfic);
                           else
                           {
                               // sinon, message d'erreur
                               effpage ();
                   // "Seuls les mails en attente d'envoi peuvent tre modifis"
                               affiche_msg ("MAIL_NON_MODIF");
                               sleep (2);
                           }

                           affpage ();
                           erreurs = 0;
                           break;

            case '/'     : // recherche d'une chaine de caractres
            case '?'     : recherche (car);
                           erreurs = 0;

                           // sortie du programme
            case ESC     : break;

                           // sortie aussi sur :q
            case ':'     : car = leccar ();

                           if (car == 'q')
                           {
                               car = ESC;
                               break;
                           }

            default      : putchar (7);  // bip

                           // si trop d'erreurs, afficher l'aide
                           if (++erreurs == 5)
                               ungetc (0, stdin);  // on ira sur l'aide
        }
    }
    while (car != ESC && nb_lignes > 0);

    // descendre en bas de page (pour sortir proprement du programme)
    while (lignecran++ < lignepage && lignecour++ < nb_lignes)
        putchar ('\n');

    // retour  la configuration standard du clavier
    mode_normal ();
}


/* affiche les caractristiques d'un mail */

void affligne (int numlig)
{
    // la largeur des diffrents champs affichs est calcule
    //  partir de la largeur de la fentre d'affichage
    // on utilise des variables statiques pour ne faire le
    // calcul qu'en cas de ncessit
    static int old_colonpage = -1; // pour dtecter changement largeur de page
    static int sz_nom, sz_sujet;   // longueur des diffrentes zones affiches
    int tot;  // pour calcul
    int i;    // simple compteur


    // si la largeur de la zone d'affichage a chang
    if (colonpage != old_colonpage)
    {
        // rpartir la largeur d'affichage entre les champs
        tot = colonpage - sz_date;

        if (tot > szmax_nom + szmax_sujet)
            tot = szmax_nom + szmax_sujet;

        sz_nom = (tot / 4) + 3;
        sz_sujet = tot - sz_nom - 2;

        // cette mmorisation vitera de refaire le calcul  chaque ligne
        old_colonpage = colonpage;
    }

    // afficher en surbrillance les mails non lus
    // (le test de la lettre  n  comme 10me caractre n'est utile que pour
    // assurer la compatibilit en lecture avec Micro$oft Internet mail)
    if (fichmail [numlig]->nomfic [8] == '.' &&
           tolower (fichmail [numlig]->nomfic [9]) == 'n')
        clair ();

    // affichage du nom de l'expditeur
    for (i = 0; i < sz_nom; i++)
        putchar (fichmail [numlig]->nom [i]);

    putchar (' ');

    // affichage du sujet
    for (i = 0; i < sz_sujet; i++)
    {
        // si prsence d'une tabulation
        if (fichmail [numlig]->sujet [i] == '\t')
            // afficher un blanc pour une meilleure mise en page
            putchar (' ');
        else
            // sinon affichage du caractre
            putchar (fichmail [numlig]->sujet [i]);
    }

    putchar (' ');

    // affichage de la date et de l'heure (sans fuseau horaire)
    printf ("%s", fichmail [numlig]->date);

    // retour  l'affichage normal
    lumnor ();
}



/* supprime de la liste un mail que l'on vient de dtruire */

void suprefmail ()
{
    int i;  // compteur


    // on l'enlve de la liste  afficher
    for (i = lignecour + 1; i < nb_lignes; i++)
        fichmail [i - 1] = fichmail [i];

    nb_lignes --;

    // on reste positionn sur un mail voisin
    if (lignecour == nb_lignes)
        monte (1);
}



/* ractualiser l'affichage de la liste des mails

   Cette fonction ne sert que dans le cas particulier d'un
   mail rajout alors qu'on est dans le rpertoire d'envoi.
   La mthode de mise  jour n'a pas besoin d'tre optimise.
   On se contente d'effacer l'ancienne liste et de la recrer. */

void majliste_mails ()
{
    // librer la mmoire occupe par l'ancienne liste
    while (nb_lignes > 0)
        free (fichmail [--nb_lignes]);

    // rcuprer la liste des fichiers mail  envoyer
    lister_mails ();

    // et les trier par date
    tridate ();
}



/* recherche d'une chaine de caractres dans la liste des mails */

void recherche (int sens)
{
    // chaine  rechercher, on utilise une variable statique
    // pour garder sa valeur aux appels suivants de la fonction
    static char chaine [80];
    int car;    // caractre lu au clavier
    int limite; // ligne marquant une extrmit de la liste
    int trouve; // indicateur : ligne contenant la chaine trouve
    int i, j;   // compteurs


    // descendre en bas de page et effacer la dernire ligne
    baspage ();
    effligne ();

    // saisir la chaine  rechercher
    putchar (sens);
    car = leccar ();

    // si chaine vide, on conserve la prcdente
    if (car != '\n')
    {
        // sinon on mmorise la nouvelle chaine
        i = 0;

        do
        {
            // cas particulier : caractre d'effacement
            if (car == RECULE || car == EFFCAR)
            {
                if (i > 0)
                {
                    putchar (CTRH);
                    putchar (' ');
                    putchar (CTRH);
                    i--;
                }
            }
            // cas gnral : affichage et mmorisation d'un caractre
            else if (i < sizeof (chaine))
            {
                putchar (car);
                chaine [i++] = car;
            }

            // lire le caractre suivant
            car = leccar ();
        }
        // jusqu' ce que chaine entirement saisie
        while (car != '\n');

        // terminer la chaine
        chaine [i] = '\0';
    }

    // initialisation variables pour recherche
    if (sens == '/')
    {
        sens = 1;
        limite = nb_lignes - 1;
    }
    else
    {
        sens = -1;
        limite = 0;
    }

    // dbut de la recherche  partir d'une ligne voisine de la ligne courante
    i = lignecour;

    // rpeter
    do
    {
        // si on n'a pas atteint l'extrmit de la liste des mails
        if (i != limite)
            // descendre ou remonter d'une ligne selon le sens de recherche
            i = i + sens;
        else
            // sinon aller  l'autre extrmit de la liste
            i = nb_lignes - 1 - limite;

        // chercher si la chaine est prsente dans la nouvelle ligne
        trouve = dansligne (chaine, i);
    }
    // jusque chaine de caractres trouve ou toute la liste explore
    while (! dansligne (chaine, i) && i != lignecour);

    // si chaine non trouve
    if (! trouve)
    {
        // afficher un message d'erreur
        // " Chaine non trouve"
        printf (message ("CHAINE_ABSENTE"));
        putchar (7);
    }

    // revenir en dbut de ligne
    putchar ('\r');

    // remonter  la ligne sur laquelle on tait
    j = lignepage;

    while (j-- > lignecran)
        montecurs ();

    // si chaine trouve
    if (trouve)
    {
        // se positionner sur la ligne de la chaine s'il y a lieu
        if (i < lignecour)
            monte (lignecour - i);
        else if (i > lignecour)
            descend (i - lignecour);

        // rafficher la ligne contenant la chaine dans une autre couleur
        putseq ("33m");
        affligne (lignecour);
        putchar ('\r');
        lumnor ();
    }
}



/* vrification de la prsence d'une chaine dans la numligime ligne */

int dansligne (char *chaine, int numlig)
{
    char *maligne; // pour viter de trop manipuler des indices de tableau
    int i, j;      // simples compteurs


    // initialisation
    i = 0;

    // le nom de l'expditeur/destinataire, le sujet et la
    // date sont regroups sur une mme chaine de caractres
    maligne = (char*) &fichmail [numlig]->nom;

    // tantque ligne non explore en entier
    while (maligne [i])
    {
        // si caractre courant = premier caractre de la chaine
        if (maligne [i] == *chaine)
        {
            // vrifier la concordance des caractres suivants
            j = 1;

            while (chaine [j] && (chaine [j] == maligne [i+j]))
                j++;

            // si chaine trouve, sortir de la fonction
            if (! chaine [j])
                return 1;
        }

        // sinon, essayer avec la suite de la ligne
        i++;
    }

    // chaine non trouve dans la ligne
    return 0;
}
